/*
 * ==========================================================================================
 * ceckconfd.c
 *
 * Date: 10 May 2004 
 * Author: Jerome Delamarche
 * Version: 1.3 
 *
 * ------------------------------------------------------------------------------------------
 * CHANGES:
 *
 * ------------------------------------------------------------------------------------------
 *
 * TODO: 
 *
 * ------------------------------------------------------------------------------------------
 *
 * Chedck the XML configuration file specified on the command line and outputs errors.
 * It is recommanded to test XML files make changes available for the module using
 * the periodic Reload feature.
 *
 * Example usage:
 *
 *	checkconf [-d] xmlfile
 * 
 * -d: debug mode
 *
 * ==========================================================================================
 */
#define APACHE_EMULATE	1
#include <sys/time.h>
#include <sys/un.h>

#include "conf.h"
#include "rules.h"

static const char *rcsid = "$Id$";
static int g_traceDebug = 0;

/*
 * --------------
 * Error Callback: 
 * --------------
 */
static void Error(const char *format, ...)
{
	va_list ap;
	va_start(ap,format);
	vfprintf(stderr,format,ap);
	va_end(ap);
	fputs("\n",stderr);
}

void Usage(int argc, char *argv[])
{
	fprintf(stderr,"%s: [-d] xmlfile\n",argv[0]);
	fprintf(stderr,"-d: debug mode\n");
	fprintf(stderr,"\n");
	exit(1);
}

/* 
 * ---------------------------------------------------------------
 * Main function
 * ---------------------------------------------------------------
 */
int main(int argc, char *argv[])
{
	xmlDocPtr	pDoc;
	guard_ctxt_t	*cfg;
	char		*cmd = NULL;
	int		i = 1;

	while (i < argc) {
		if (!strcmp(argv[i],"-d")) {
			g_traceDebug = 1; 
			i++;
			continue; 
		}
		cmd = argv[i]; 
		i++;
	}
	if (!cmd) Usage(argc,argv);
	
	InitConf(Error);
	if (!(pDoc = ReadConfFile(cmd))) {
		fprintf(stderr,"Configuration Error\n");
		return -1;
	}

	/* Check the conf XML file semantics: */
	cfg = InitRules(g_traceDebug,cmd,Error);
	if (BuildRules(cfg,pDoc)) {
		fprintf(stderr,"Incorrect Rules\n");
		return -2;
	}

	if (g_traceDebug) DumpRules(cfg);

	EndConf();	/* necessary to free the memory allocated by libxml */
	EndRules(cfg);

	return 0;
}

/*
 * --------------------------------------------------------
 * Memory handling functions that emulate the Apache's ones:
 * --------------------------------------------------------
 */
void *APR_PALLOC(void *p, unsigned int size) 
{
	return (void *)malloc(size);
}

void *APR_PSTRDUP(void *p, const char *s)
{
	return strdup(s);
}

ap_regex_t *AP_PREGCOMP(void *p, const char *pat, int fl)
{
	ap_regex_t *reg = (ap_regex_t *)malloc(sizeof(ap_regex_t));
	if (!regcomp(reg,pat,fl)) return reg;
	return NULL;
}

apr_status_t APR_POOL_CREATE(apr_pool_t **p) { }
void APR_POOL_DESTROY(apr_pool_t *p) { }

/* EOF */
